import osgeo.gdal as gdal
from scikits.eartho.io.partrw import *
from string import *
import os
import numpy as np

def _IngestRaster(srcDataset, dstDataset,  bandIndex):
    reader = PartReaderWriter(srcDataset, bandIndex+1)   
    writer = PartReaderWriter(dstDataset, bandIndex*3+1)   
    tile,  offsetX,  offsetY = reader.ReadNextTile()
    while not tile is None:
        writer.Write(tile,  offsetX,  offsetY)
        tile,  offsetX,  offsetY = reader.ReadNextTile()

def _IngestCoord(srcDataset, dstDataset,  bandIndex):
    geoTransform = srcDataset.GetGeoTransform()
    geoX = geoTransform[0]
    geoY = geoTransform[3]
    scaleX = geoTransform[1]
    scaleY = geoTransform[5]
    partX = PartReaderWriter(dstDataset, bandIndex*3+2)   
    partY = PartReaderWriter(dstDataset, bandIndex*3+3)   
    tile,  offsetX,  offsetY = partX.ReadNextTile()
    while not tile == None:
        lineX = geoX + (offsetX + np.array(list(range(0,  tile.shape[1])))) * scaleX
        lineY = geoY + (offsetY + np.array(list(range(0,  tile.shape[0])))) * scaleY
        tile[:, :] = lineX
        partX.Write(tile,  offsetX,  offsetY)
        tile = tile.transpose()
        tile[:, :] = lineY
        tile = tile.transpose()
        partY.Write(tile,  offsetX,  offsetY)
        tile,  offsetX,  offsetY = partX.ReadNextTile()
    


def IngestOneFile(source,  destination,  progressCallback=None):
    datasetSource = gdal.Open(source,  gdal.GA_ReadOnly)
    driver = "PCIDSK"
    ext = ".pix"
    driverDestination = gdal.GetDriverByName(driver)
    if lower(destination[-4:]) != ext:
       destination = destination + ext 
    if driverDestination is None:
        print("Failed to create driver! Please ensure GDAL is built with the %s driver enabled."  % driver)
        return None
    try:
        os.remove(destination)    
    except:
        pass
    progressCallback("Creating/overwriting file: %s" % destination,  0, 100)
    datasetDestination = driverDestination.Create(destination,  datasetSource.RasterXSize, datasetSource.RasterYSize, datasetSource.RasterCount * 3,  gdal.GDT_Float32) 
    datasetDestination.SetGeoTransform(datasetSource.GetGeoTransform())
    datasetDestination.SetProjection(datasetSource.GetProjection())
    for b in range(0,  datasetSource.RasterCount):
        if not progressCallback is None:
            progressCallback("Ingesting band %d of %d" % (b+1, datasetSource.RasterCount),  b, datasetSource.RasterCount)
        spectPart = datasetDestination.GetRasterBand(b*3 + 1)   
        #spectPart.SetMetadataItem("scikits.eartho.bandPart",  "SPECT" ,  "")   
        spectPart.SetMetadataItem("scikits.eartho.bandIndex",  "%d" % b,  "")   
        spectPart.SetDescription("Spectral data for band %d from %s" % (b+1,  source))
        coordPartX = datasetDestination.GetRasterBand(b*3 + 2)   
        coordPartX.SetMetadataItem("scikits.eartho.bandPart",  "XCOORD" ,  "")   
        coordPartX.SetMetadataItem("scikits.eartho.bandIndex",  "%d" % b,  "")   
        coordPartX.SetDescription("X Coordinates for band %d from %s" % (b+1,  source))
        coordPartY = datasetDestination.GetRasterBand(b*3 + 3)   
        coordPartY.SetMetadataItem("scikits.eartho.bandPart",  "YCOORD" ,  "")   
        coordPartY.SetMetadataItem("scikits.eartho.bandIndex",  "%d" % b,  "")   
        coordPartY.SetDescription("Y Coordinates for band %d from %s" % (b+1,  source))
        _IngestRaster(datasetSource, datasetDestination,  b)
        _IngestCoord(datasetSource, datasetDestination,  b)
    progressCallback("Ingestion complete.",  100, 100)
    return destination
    
    
    

