import osgeo.gdal as gdal

class PartReaderWriter:
    def __init__(self,  dataset,  bandNum=1):
        if type(dataset) == str:
            self.dataset = gdal.Open(dataset, gdal.GA_ReadOnly)
        else:
            self.dataset = dataset
        self.raster = self.dataset.GetRasterBand(bandNum)
        self.tileWidth =1024
        self.tileHeight = 1024
        self.tileNum = 0 
    
    def SetTileSize(self,  width,  height):
        self.tileWidth =width
        self.tileHeight = height

    def ResetIterator(self):
        self.tileNum = 0
        
    def ReadNextTile(self):
        result = self.ReadTile(self.tileNum)
        if result[0] is None:
            return None,  -1,  -1
        tileRaster,  offsetX,  offsetY  = result
        self.tileNum = self.tileNum + 1
        return tileRaster,  offsetX,  offsetY
        
    def ReadTile(self,  tileNum):
        tilesInWidth = self.raster.XSize / self.tileWidth + 1
        offsetX = tileNum % tilesInWidth * self.tileWidth
        offsetY = tileNum / tilesInWidth * self.tileHeight
        width = min(self.tileWidth,  self.raster.XSize - offsetX) 
        height = min(self.tileHeight,  self.raster.YSize - offsetY) 
        if width <= 0 or height <= 0:
            return None,  -1,  -1
        tileRaster = self.raster.ReadAsArray(offsetX,  offsetY, width,  height)
        return tileRaster,  offsetX,  offsetY
        
    def Write(self, array, offsetX,  offsetY):
        self.raster.WriteArray(array,  offsetX,  offsetY)
    
