from .bandrw import *

class SpatialQuery:
    def __init__(self,  band,  ulX, ulY,  lrX,  lrY):
        self.band = band
        self.ulX = ulX
        self.ulY = ulY
        self.lrX = lrX
        self.lrY = lrY
        self.Run()
    
    def Run(self):
        self.band.xCoordPart.ResetIterator()
        HUGE = 100000000
        minP = HUGE
        minL = HUGE
        maxP = -HUGE
        maxL = -HUGE
        valid = False
        tileRasterX,  offsetX,  offsetY = self.band.xCoordPart.ReadNextTile()
        while not tileRasterX is None:
            if self._TileRelevant(tileRasterX,  self.ulX,  self.lrX):
                tileMinP,  tileMinL,  tileMaxP,  tileMaxL = self._ForTile(tileRasterX,  self.ulX,  self.lrX)
                tileMinP += offsetX
                tileMinL += offsetY
                tileMaxP += offsetX
                tileMaxL += offsetY
                if tileMinP < minP:
                    minP = tileMinP
                    valid = True
                if tileMinL < minL:
                    minL = tileMinL
                    valid = True
                if tileMaxP > maxP:
                    maxP = tileMaxP
                    valid = True
                if tileMaxL > maxL:
                    maxL = tileMaxL
                    valid = True
            tileRasterX,  offsetX,  offsetY = self.band.xCoordPart.ReadNextTile()
        tileRasterY,  offsetX,  offsetY = self.band.yCoordPart.ReadNextTile()
        minY = min(self.ulY,  self.lrY)  # Y direction of change may be upside down depending on projection
        maxY = max(self.ulY,  self.lrY)
        while not tileRasterY is None:
            if self._TileRelevant(tileRasterY, minY,  maxY):
                tileMinP,  tileMinL,  tileMaxP,  tileMaxL = self._ForTile(tileRasterY,  minY,  maxY)
                tileMinP += offsetX
                tileMinL += offsetY
                tileMaxP += offsetX
                tileMaxL += offsetY
                if tileMinP < minP:
                    minP = tileMinP
                    valid = True
                if tileMinL < minL:
                    minL = tileMinL
                    valid = True
                if tileMaxP > maxP:
                    maxP = tileMaxP
                    valid = True
                if tileMaxL > maxL:
                    maxL = tileMaxL
                    valid = True
            tileRasterY,  offsetX,  offsetY = self.band.yCoordPart.ReadNextTile()
        if valid:    
            self.result = [minP,  minL,  maxP,  maxL]
        else:
            self.result = None
        

    def _TileRelevant(self,  tileRaster,  min,  max):
        return ((tileRaster >= min).__and__(tileRaster <= max)).any()     #TODO: can be optimized

    def _ForTile(self,  tileRaster,  min,  max):
        tileMinP = None
        tileMinL = None
        tileMaxP = None
        tileMaxL = None
        current = 0
        while tileMinL is None and current < tileRaster.shape[0]:
            if ((tileRaster[current, :] >= min) and (tileRaster[current, :] <= max)).any():
                tileMinL = current
            current += 1   
        current = tileRaster.shape[0] - 1
        while tileMaxL is None and current >= 0:
            if ((tileRaster[current, :] >= min) and (tileRaster[current, :] <= max)).any():
                tileMaxL = current
            current -= 1   
        current = 0
        while tileMinP is None and current < tileRaster.shape[1]:
            if ((tileRaster[:, current] >= min) and (tileRaster[:, current] <= max)).any():
                tileMinP = current
            current += 1    
        current = tileRaster.shape[1] - 1 
        while tileMaxP is None and current >= 0:
            if ((tileRaster[:, current] >= min) and (tileRaster[:, current] <= max)).any():
                tileMaxP = current
            current -= 1   
            print(tileMinP,  tileMinL,  tileMaxP,  tileMaxL)
        return  tileMinP,  tileMinL,  tileMaxP,  tileMaxL
